# Getting started

### 1. Load the library
````
var pgpLib = require('pg-promise');
````
### 2. Configure connection parameters
````
var config = {
    host: 'localhost',
    port: 5432,
    database: 'my_db_name',
    user: 'postgres',
    password: 'bla-bla'
};
````
<b>NOTE:</b> The library itself doesn't use this object at all, just passing it on to PG to interpret and use. See ````ConnectionParameters```` in PG package for details.
### 3. Initialize the library
````
var pgp = pgpLib(config);
````
Only one global instance should be used as shown above. Now you are ready to use it.

# Usage
### The basics
In order to eliminate the chances of unexpected query results and make code more robust, each request is parametrized with the expected/supported return result, using type ````queryResult```` as shown below:
````
queryResult = {
    one: 1,
    many: 2,
    none: 4
};
````
In the following generic-query example we indicate that the call must return either no records or multiple records:
````
pgp.query("select * from users", queryResult.none | queryResult.many);
````
This usage pattern is facilitated through result-specific methods that can be used instead of the generic query:
````
pgp.many("select * from users"); // multiple records are expected
pgp.one("select * from users limit 1"); // one record is expected
pgp.none("update users set active=TRUE where id=1"); // no records expected
````
There are also mixed-result ones, called ````oneOrNone```` and ````manyOrNone````. And when the expected and actual results do not match, the call will be rejected.

Each of the query calls returns a Promise object to be used in the standard way:
````
pgp.many("select * from users").then(function(data){
    console.log(data); // printing the data received
},function(reason){
    console.log(reason); // printing the reason why the call was rejected
});
````
### Functions and Procedures
In PostgreSQL stored procedures are just functions that usually do not return anything.

Suppose we want to call a function to find audit records by user id and maximum time stamp. We can call a corresponding function as shown below:
````
pgp.func('findAudit', [
    123,
    new Date()
]);
````
We passed it user Id = 123, plus current Date/Time as the time stamp. We assume that the function signature matches the the parameters that we passed. All values passed are serialized automatically to comply with PostgreSQL requirements.
In the same way you can call ````pgp.proc````, which doesn't care about the returned result. Both methods return a promise object.

### Transactions
Every call shown in the chapter above would acquire a new connection from the pool and release it when done. In order to execute a transaction on the same connection, a transaction class is to be used.

Example:
````
var promise = require('promise');

var tx = new pgp.tx(); // creating a new transaction object
tx.exec(function(){
    return promise.all([
        tx.none("update users set active=TRUE"),
        tx.one("insert into audit(event) values('user changed') returning id")
    ]);
}).then(function(data){
    console.log(data); // printing successful transation output
}, function(reason){
    console.log(reason); // printing the reason why the transaction was rejected
});
````
In the above example we create a new transaction object and call its method ````exec````, passing it a call-back function that must do all the queries needed and return a promise object. In the example we use ````promise.all```` to indicate that we want both queries inside the transaction to succeed to consider it a success; otherwise the transaction is to be rolled back.

Note that while inside a transaction, we make calls to the same-named methods as outside of transactions, except on the transaction object instance, instead of the global ````pgp```` object, which gives it access to the shared transaction connection object. The same goes for calling functions and procedures within transactions, using ````tx.func```` and ````tx.proc```` accordingly.

### Type Helpers
The library provides several basic helper functions to convert a basic type into proper PostgreSQL presentation that can be used to pass directly into queries or functions as parameters.
All of such helper functions are located within namespace ````pgp.as````:
````
pgp.as.bool(value); // reuturns proper postgresql boolean presentation
pgp.as.text(value); // reuturns proper postgresql text presentation, fixing quota symbols
pgp.as.date(value); // reuturns proper postgresql date/time presentation
````
These helpers are not associated with a connection, and can be called from inside or outside a transaction.

# Advanced
Work in progress. This chapter will be written just as the first version is finished.
